@@ -167,7 +167,6 @@ static void       gimp_drawable_real_update        (GimpDrawable      *drawable,
                                                     gint               width,
                                                     gint               height);
 
-static void    gimp_drawable_real_filters_changed  (GimpDrawable      *drawable);
 static gint64  gimp_drawable_real_estimate_memsize (GimpDrawable      *drawable,
                                                     GimpComponentType  component_type,
                                                     gint               width,
@@ -311,7 +310,7 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
   klass->format_changed           = NULL;
   klass->alpha_changed            = NULL;
   klass->bounding_box_changed     = NULL;
-  klass->filters_changed          = gimp_drawable_real_filters_changed;
+  klass->filters_changed          = NULL;
   klass->estimate_memsize         = gimp_drawable_real_estimate_memsize;
   klass->update_all               = gimp_drawable_real_update_all;
   klass->invalidate_boundary      = NULL;
@@ -581,7 +580,7 @@ gimp_drawable_scale (GimpItem              *item,
           if (GIMP_IS_DRAWABLE_FILTER (list->data))
             {
               GimpDrawableFilter *filter = list->data;
-              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
+              GimpChannel        *mask   = gimp_drawable_filter_get_mask (filter);
               GeglRectangle      *rect   = GEGL_RECTANGLE (0, 0,
                                                            new_width,
                                                            new_height);
@@ -690,7 +689,7 @@ gimp_drawable_resize (GimpItem     *item,
           if (GIMP_IS_DRAWABLE_FILTER (list->data))
             {
               GimpDrawableFilter *filter = list->data;
-              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
+              GimpChannel        *mask   = gimp_drawable_filter_get_mask (filter);
               GeglRectangle       rect   = {0, 0, new_width, new_height};
 
               /* Don't resize partial layer effects */
@@ -764,30 +763,6 @@ gimp_drawable_rotate (GimpItem         *item,
                                      new_off_x, new_off_y, FALSE);
       g_object_unref (buffer);
     }
-
-  if (GIMP_IS_LAYER (drawable))
-    {
-      GList *list;
-      gint   width  = gimp_item_get_width (GIMP_ITEM (drawable));
-      gint   height = gimp_item_get_height (GIMP_ITEM (drawable));
-
-      for (list = GIMP_LIST (drawable->private->filter_stack)->queue->tail;
-           list; list = g_list_previous (list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (list->data))
-            {
-              GimpDrawableFilter *filter = list->data;
-              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
-              GeglRectangle       rect   = {0, 0, width, height};
-
-              /* Don't resize partial layer effects */
-              if (gimp_channel_is_empty (mask))
-                gimp_drawable_filter_refresh_crop (filter, &rect);
-            }
-        }
-      if (list)
-        g_list_free (list);
-    }
 }
 
 static void
@@ -896,12 +871,6 @@ gimp_drawable_real_update (GimpDrawable *drawable,
   gimp_viewable_invalidate_preview (GIMP_VIEWABLE (drawable));
 }
 
-static void
-gimp_drawable_real_filters_changed (GimpDrawable *drawable)
-{
-  gimp_drawable_update_bounding_box (drawable);
-}
-
 static gint64
 gimp_drawable_real_estimate_memsize (GimpDrawable      *drawable,
                                      GimpComponentType  component_type,
@@ -1361,11 +1330,10 @@ gimp_drawable_convert_type (GimpDrawable      *drawable,
                             gboolean           push_undo,
                             GimpProgress      *progress)
 {
-  const Babl    *old_format;
-  const Babl    *new_format;
-  gint           old_bits;
-  gint           new_bits;
-  GimpContainer *filters;
+  const Babl *old_format;
+  const Babl *new_format;
+  gint        old_bits;
+  gint        new_bits;
 
   g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
   g_return_if_fail (GIMP_IS_IMAGE (dest_image));
@@ -1411,44 +1379,6 @@ gimp_drawable_convert_type (GimpDrawable      *drawable,
                                                     push_undo,
                                                     progress);
 
-  /* Update the masks of any filters */
-  /* TODO: Move to gimp_drawable_real_convert_type () once it's updated
-   * to run for all GimpDrawable child classes */
-  filters = gimp_drawable_get_filters (drawable);
-  if (gimp_container_get_n_children (filters) > 0)
-    {
-      const Babl    *mask_format;
-      GList         *filter_list;
-      GimpPrecision  new_mask_precision;
-
-      mask_format        = gimp_image_get_mask_format (dest_image);
-      new_mask_precision = gimp_babl_format_get_precision (mask_format);
-
-      for (filter_list = GIMP_LIST (filters)->queue->tail;
-           filter_list;
-           filter_list = g_list_previous (filter_list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-            {
-              GimpDrawableFilter     *filter = filter_list->data;
-              GimpDrawableFilterMask *mask;
-
-              mask = gimp_drawable_filter_get_mask (filter);
-
-              if (new_mask_precision == gimp_drawable_get_precision (GIMP_DRAWABLE (mask)))
-                /* The only change which may happen for masks is the bit-depth. */
-                break;
-
-              gimp_drawable_convert_type (GIMP_DRAWABLE (mask), dest_image,
-                                          GIMP_GRAY, new_mask_precision,
-                                          FALSE,
-                                          NULL, NULL,
-                                          layer_dither_type, mask_dither_type,
-                                          push_undo, progress);
-            }
-        }
-    }
-
   if (progress)
     gimp_progress_set_value (progress, 1.0);
 }
@@ -1579,7 +1509,7 @@ gimp_drawable_get_buffer_with_effects (GimpDrawable *drawable)
 
   if (drawable->private->paint_count == 0)
     {
-      if (gimp_drawable_has_visible_filters (drawable))
+      if (gimp_drawable_has_filters (drawable))
         {
           GeglNode                *source = NULL;
           GeglBuffer              *buffer;
@@ -1607,17 +1537,17 @@ gimp_drawable_get_buffer_with_effects (GimpDrawable *drawable)
             }
           else
             {
-              return g_object_ref (buffer);
+              return buffer;
             }
         }
       else
         {
-          return g_object_ref (GIMP_DRAWABLE_GET_CLASS (drawable)->get_buffer (drawable));
+          return GIMP_DRAWABLE_GET_CLASS (drawable)->get_buffer (drawable);
         }
     }
   else
     {
-      return g_object_ref (drawable->private->paint_buffer);
+      return drawable->private->paint_buffer;
     }
 }
 
@@ -2107,7 +2037,7 @@ gimp_drawable_end_paint (GimpDrawable *drawable)
   drawable->private->paint_count--;
 
   /* Refresh filters after painting */
-  if (gimp_drawable_has_visible_filters (drawable) &&
+  if (gimp_drawable_has_filters (drawable) &&
       drawable->private->paint_count == 0)
     {
       gimp_item_set_visible (GIMP_ITEM (drawable), FALSE, FALSE);
