@@ -36,7 +36,7 @@
 #include "gimplayer.h"
 #include "gimpmarshal.h"
 
-#include "vectors/gimppath.h"
+#include "vectors/gimpvectors.h"
 
 #include "gimp-intl.h"
 
@@ -187,8 +187,8 @@ gimp_item_list_constructed (GObject *object)
 
   gimp_assert (GIMP_IS_IMAGE (set->p->image));
   gimp_assert (set->p->item_type == GIMP_TYPE_LAYER   ||
-               set->p->item_type == GIMP_TYPE_CHANNEL ||
-               set->p->item_type == GIMP_TYPE_PATH);
+               set->p->item_type == GIMP_TYPE_VECTORS ||
+               set->p->item_type == GIMP_TYPE_CHANNEL);
 
   if (! set->p->is_pattern)
     {
@@ -196,11 +196,10 @@ gimp_item_list_constructed (GObject *object)
 
       if (set->p->item_type == GIMP_TYPE_LAYER)
         container = gimp_image_get_layers (set->p->image);
-      else if  (set->p->item_type == GIMP_TYPE_CHANNEL)
-        container = gimp_image_get_channels (set->p->image);
+      else if (set->p->item_type == GIMP_TYPE_VECTORS)
+        container = gimp_image_get_vectors (set->p->image);
       else
-        container = gimp_image_get_paths (set->p->image);
-
+        container = gimp_image_get_channels (set->p->image);
       g_signal_connect (container, "remove",
                         G_CALLBACK (gimp_item_list_item_remove),
                         set);
@@ -221,11 +220,10 @@ gimp_item_list_dispose (GObject *object)
 
       if (set->p->item_type == GIMP_TYPE_LAYER)
         container = gimp_image_get_layers (set->p->image);
-      else if (set->p->item_type == GIMP_TYPE_CHANNEL)
-        container = gimp_image_get_channels (set->p->image);
+      else if (set->p->item_type == GIMP_TYPE_VECTORS)
+        container = gimp_image_get_vectors (set->p->image);
       else
-        container = gimp_image_get_paths (set->p->image);
-
+        container = gimp_image_get_channels (set->p->image);
       g_signal_handlers_disconnect_by_func (container,
                                             G_CALLBACK (gimp_item_list_item_remove),
                                             set);
@@ -351,10 +349,10 @@ gimp_item_list_named_new (GimpImage   *image,
     {
       if (item_type == GIMP_TYPE_LAYER)
         items = gimp_image_get_selected_layers (image);
+      else if (item_type == GIMP_TYPE_VECTORS)
+        items = gimp_image_get_selected_vectors (image);
       else if (item_type == GIMP_TYPE_CHANNEL)
         items = gimp_image_get_selected_channels (image);
-      else if (item_type == GIMP_TYPE_PATH)
-        items = gimp_image_get_selected_paths (image);
 
       if (! items)
         return NULL;
@@ -557,7 +555,7 @@ gimp_item_list_get_items_by_substr (GimpItemList  *set,
                                     const gchar   *pattern,
                                     GError       **error)
 {
-  GList  *items = NULL;
+  GList  *items;
   GList  *match = NULL;
   GList  *iter;
 
@@ -568,11 +566,15 @@ gimp_item_list_get_items_by_substr (GimpItemList  *set,
     return NULL;
 
   if (set->p->item_type == GIMP_TYPE_LAYER)
-    items = gimp_image_get_layer_list (set->p->image);
-  else if (set->p->item_type == GIMP_TYPE_CHANNEL)
-    items = gimp_image_get_channel_list (set->p->image);
-  else if (set->p->item_type == GIMP_TYPE_PATH)
-    items = gimp_image_get_path_list (set->p->image);
+    {
+      items = gimp_image_get_layer_list (set->p->image);
+    }
+  else
+    {
+      g_critical ("%s: only list of GimpLayer supported for now.",
+                  G_STRFUNC);
+      return NULL;
+    }
 
   for (iter = items; iter; iter = iter->next)
     {
@@ -602,7 +604,7 @@ gimp_item_list_get_items_by_glob (GimpItemList  *set,
                                   const gchar   *pattern,
                                   GError       **error)
 {
-  GList        *items = NULL;
+  GList        *items;
   GList        *match = NULL;
   GList        *iter;
   GPatternSpec *spec;
@@ -614,11 +616,15 @@ gimp_item_list_get_items_by_glob (GimpItemList  *set,
     return NULL;
 
   if (set->p->item_type == GIMP_TYPE_LAYER)
-    items = gimp_image_get_layer_list (set->p->image);
-  else if (set->p->item_type == GIMP_TYPE_CHANNEL)
-    items = gimp_image_get_channel_list (set->p->image);
-  else if (set->p->item_type == GIMP_TYPE_PATH)
-    items = gimp_image_get_path_list (set->p->image);
+    {
+      items = gimp_image_get_layer_list (set->p->image);
+    }
+  else
+    {
+      g_critical ("%s: only list of GimpLayer supported for now.",
+                  G_STRFUNC);
+      return NULL;
+    }
 
   spec = g_pattern_spec_new (pattern);
   for (iter = items; iter; iter = iter->next)
@@ -650,7 +656,7 @@ gimp_item_list_get_items_by_regexp (GimpItemList  *set,
                                     const gchar   *pattern,
                                     GError       **error)
 {
-  GList  *items = NULL;
+  GList  *items;
   GList  *match = NULL;
   GList  *iter;
   GRegex *regex;
@@ -665,11 +671,15 @@ gimp_item_list_get_items_by_regexp (GimpItemList  *set,
     return NULL;
 
   if (set->p->item_type == GIMP_TYPE_LAYER)
-    items = gimp_image_get_layer_list (set->p->image);
-  else if (set->p->item_type == GIMP_TYPE_CHANNEL)
-    items = gimp_image_get_channel_list (set->p->image);
-  else if (set->p->item_type == GIMP_TYPE_PATH)
-    items = gimp_image_get_path_list (set->p->image);
+    {
+      items = gimp_image_get_layer_list (set->p->image);
+    }
+  else
+    {
+      g_critical ("%s: only list of GimpLayer supported for now.",
+                  G_STRFUNC);
+      return NULL;
+    }
 
   for (iter = items; iter; iter = iter->next)
     {
