@@ -58,13 +58,13 @@ typedef struct
 } PreviewSettings;
 
 
-struct _GimpDrawablePreview
+struct _GimpDrawablePreviewPrivate
 {
-  GimpScrolledPreview  parent_instance;
-
-  GimpDrawable        *drawable;
+  GimpDrawable *drawable;
 };
 
+#define GET_PRIVATE(obj) (((GimpDrawablePreview *) (obj))->priv)
+
 
 static void  gimp_drawable_preview_constructed   (GObject         *object);
 static void  gimp_drawable_preview_dispose       (GObject         *object);
@@ -92,7 +92,8 @@ static void  gimp_drawable_preview_set_drawable  (GimpDrawablePreview *preview,
                                                   GimpDrawable        *drawable);
 
 
-G_DEFINE_TYPE (GimpDrawablePreview, gimp_drawable_preview, GIMP_TYPE_SCROLLED_PREVIEW)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpDrawablePreview, gimp_drawable_preview,
+                            GIMP_TYPE_SCROLLED_PREVIEW)
 
 #define parent_class gimp_drawable_preview_parent_class
 
@@ -137,6 +138,8 @@ gimp_drawable_preview_class_init (GimpDrawablePreviewClass *klass)
 static void
 gimp_drawable_preview_init (GimpDrawablePreview *preview)
 {
+  preview->priv = gimp_drawable_preview_get_instance_private (preview);
+
   g_object_set (gimp_preview_get_area (GIMP_PREVIEW (preview)),
                 "check-size", gimp_check_size (),
                 "check-type", gimp_check_type (),
@@ -175,8 +178,8 @@ gimp_drawable_preview_constructed (GObject *object)
 static void
 gimp_drawable_preview_dispose (GObject *object)
 {
-  GimpDrawablePreview *preview = GIMP_DRAWABLE_PREVIEW (object);
-  const gchar         *data_name;
+  GimpDrawablePreviewPrivate *priv = GET_PRIVATE (object);
+  const gchar                *data_name;
 
   data_name = g_object_get_data (G_OBJECT (object),
                                  "gimp-drawable-preview-data-name");
@@ -194,7 +197,7 @@ gimp_drawable_preview_dispose (GObject *object)
       g_bytes_unref (bytes);
     }
 
-  g_clear_object (&preview->drawable);
+  g_clear_object (&priv->drawable);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -270,17 +273,17 @@ gimp_drawable_preview_style_updated (GtkWidget *widget)
 static void
 gimp_drawable_preview_draw_original (GimpPreview *preview)
 {
-  GimpDrawablePreview *drawable_preview = GIMP_DRAWABLE_PREVIEW (preview);
-  GBytes              *buffer;
-  gint                 width, height;
-  gint                 tn_width, tn_height;
-  gint                 xoff, yoff;
-  gint                 xmin, ymin;
-  gint                 xmax, ymax;
-  gint                 bpp;
-  GimpImageType        type;
+  GimpDrawablePreviewPrivate *priv = GET_PRIVATE (preview);
+  GBytes                     *buffer;
+  gint                        width, height;
+  gint                        tn_width, tn_height;
+  gint                        xoff, yoff;
+  gint                        xmin, ymin;
+  gint                        xmax, ymax;
+  gint                        bpp;
+  GimpImageType               type;
 
-  if (drawable_preview->drawable == NULL)
+  if (priv->drawable == NULL)
     return;
 
   gimp_preview_get_size (preview, &width, &height);
@@ -292,7 +295,7 @@ gimp_drawable_preview_draw_original (GimpPreview *preview)
 
   gimp_preview_set_offsets (preview, xoff, yoff);
 
-  buffer = gimp_drawable_get_sub_thumbnail_data (drawable_preview->drawable,
+  buffer = gimp_drawable_get_sub_thumbnail_data (priv->drawable,
                                                  xoff + xmin,
                                                  yoff + ymin,
                                                  width, height,
@@ -322,10 +325,10 @@ gimp_drawable_preview_draw_thumb (GimpPreview     *preview,
                                   gint             width,
                                   gint             height)
 {
-  GimpDrawablePreview *drawable_preview = GIMP_DRAWABLE_PREVIEW (preview);
+  GimpDrawablePreviewPrivate *priv = GET_PRIVATE (preview);
 
-  if (drawable_preview->drawable)
-    _gimp_drawable_preview_area_draw_thumb (area, drawable_preview->drawable,
+  if (priv->drawable)
+    _gimp_drawable_preview_area_draw_thumb (area, priv->drawable,
                                             width, height);
 }
 
@@ -420,16 +423,17 @@ gimp_drawable_preview_draw_area (GimpDrawablePreview *preview,
                                  const guchar        *buf,
                                  gint                 rowstride)
 {
-  GimpPreview *gimp_preview = GIMP_PREVIEW (preview);
-  GtkWidget   *area         = gimp_preview_get_area (gimp_preview);
-  GimpImage   *image;
-  gint         xmin, ymin;
-  gint         xoff, yoff;
+  GimpDrawablePreviewPrivate *priv         = GET_PRIVATE (preview);
+  GimpPreview                *gimp_preview = GIMP_PREVIEW (preview);
+  GtkWidget                  *area         = gimp_preview_get_area (gimp_preview);
+  GimpImage                  *image;
+  gint                        xmin, ymin;
+  gint                        xoff, yoff;
 
   gimp_preview_get_bounds (gimp_preview, &xmin, &ymin, NULL, NULL);
   gimp_preview_get_offsets (gimp_preview, &xoff, &yoff);
 
-  image = gimp_item_get_image (GIMP_ITEM (preview->drawable));
+  image = gimp_item_get_image (GIMP_ITEM (priv->drawable));
 
   if (gimp_selection_is_empty (image))
     {
@@ -438,7 +442,7 @@ gimp_drawable_preview_draw_area (GimpDrawablePreview *preview,
                               y - yoff - ymin,
                               width,
                               height,
-                              gimp_drawable_type (preview->drawable),
+                              gimp_drawable_type (priv->drawable),
                               buf, rowstride);
     }
   else
@@ -449,9 +453,9 @@ gimp_drawable_preview_draw_area (GimpDrawablePreview *preview,
       gint draw_x, draw_y;
       gint draw_width, draw_height;
 
-      gimp_drawable_get_offsets (preview->drawable, &offset_x, &offset_y);
+      gimp_drawable_get_offsets (priv->drawable, &offset_x, &offset_y);
 
-      if (gimp_drawable_mask_intersect (preview->drawable,
+      if (gimp_drawable_mask_intersect (priv->drawable,
                                         &mask_x, &mask_y,
                                         &mask_width, &mask_height) &&
           gimp_rectangle_intersect (mask_x, mask_y,
@@ -469,7 +473,7 @@ gimp_drawable_preview_draw_area (GimpDrawablePreview *preview,
 
           selection = gimp_image_get_selection (image);
 
-          src = gimp_drawable_get_sub_thumbnail_data (preview->drawable,
+          src = gimp_drawable_get_sub_thumbnail_data (priv->drawable,
                                                       draw_x, draw_y,
                                                       draw_width, draw_height,
                                                       draw_width, draw_height,
@@ -536,12 +540,13 @@ static void
 gimp_drawable_preview_set_drawable (GimpDrawablePreview *drawable_preview,
                                     GimpDrawable        *drawable)
 {
-  GimpPreview *preview = GIMP_PREVIEW (drawable_preview);
-  gint         x1, y1, x2, y2;
+  GimpPreview                *preview = GIMP_PREVIEW (drawable_preview);
+  GimpDrawablePreviewPrivate *priv    = GET_PRIVATE (preview);
+  gint                        x1, y1, x2, y2;
 
-  g_return_if_fail (drawable_preview->drawable == NULL);
+  g_return_if_fail (priv->drawable == NULL);
 
-  drawable_preview->drawable = drawable;
+  priv->drawable = drawable;
 
   _gimp_drawable_preview_get_bounds (drawable, &x1, &y1, &x2, &y2);
 
@@ -554,7 +559,7 @@ gimp_drawable_preview_set_drawable (GimpDrawablePreview *drawable_preview,
       guchar    *cmap;
       gint       num_colors;
 
-      cmap = gimp_palette_get_colormap (gimp_image_get_palette (image), babl_format ("R'G'B' u8"), &num_colors, NULL);
+      cmap = gimp_image_get_colormap (image, NULL, &num_colors);
       gimp_preview_area_set_colormap (GIMP_PREVIEW_AREA (area),
                                       cmap, num_colors);
       g_free (cmap);
@@ -634,5 +639,5 @@ gimp_drawable_preview_get_drawable (GimpDrawablePreview *preview)
 {
   g_return_val_if_fail (GIMP_IS_DRAWABLE_PREVIEW (preview), NULL);
 
-  return preview->drawable;
+  return GET_PRIVATE (preview)->drawable;
 }
